@@ -18,15 +18,15 @@
 #include "testrecurrenceexception.h"
 #include "memorycalendar.h"
 
-#include <qtest.h>
-#include <qdebug.h>
+
+#include <QTest>
 QTEST_MAIN(TestRecurrenceException)
 
 void TestRecurrenceException::testCreateTodoException()
 {
-    const KDateTime dtstart(QDate(2013, 03, 10), QTime(10, 0, 0), KDateTime::UTC);
-    const KDateTime dtdue(QDate(2013, 03, 10), QTime(11, 0, 0), KDateTime::UTC);
-    const KDateTime recurrenceId(KDateTime(dtstart).addDays(1));
+    const QDateTime dtstart(QDate(2013, 03, 10), QTime(10, 0, 0), Qt::UTC);
+    const QDateTime dtdue(QDate(2013, 03, 10), QTime(11, 0, 0), Qt::UTC);
+    const QDateTime recurrenceId(QDateTime(dtstart).addDays(1));
 
     KCalCore::Todo::Ptr todo(new KCalCore::Todo());
     todo->setUid(QStringLiteral("todo"));
@@ -35,10 +35,12 @@ void TestRecurrenceException::testCreateTodoException()
     todo->recurrence()->setDaily(1);
     todo->recurrence()->setDuration(3);
 
-    const KCalCore::Todo::Ptr exception = KCalCore::MemoryCalendar::createException(todo, recurrenceId, false).staticCast<KCalCore::Todo>();
+    const KCalCore::Todo::Ptr exception =
+        KCalCore::MemoryCalendar::createException(todo, recurrenceId, false).staticCast<KCalCore::Todo>();
     QCOMPARE(exception->dtStart(), recurrenceId);
-    QCOMPARE(exception->dtDue(), KDateTime(dtdue).addDays(1));
-    //FIXME should be done on clearing the recurrence, but we can't due to BC. Probably not that important as long as dtRecurrence is ignored if the todo is not recurring
-    //QCOMPARE(exception->dtRecurrence(), KDateTime());
+    QCOMPARE(exception->dtDue(), QDateTime(dtdue).addDays(1));
+    //FIXME should be done on clearing the recurrence, but we can't due to BC.
+    //Probably not that important as long as dtRecurrence is ignored if the todo is not recurring
+    //QCOMPARE(exception->dtRecurrence(), QDateTime());
     //TODO dtCompleted
 }
