@@ -20,8 +20,11 @@
 */
 #include "testrecurtodo.h"
 #include "todo.h"
-#include <qdebug.h>
-#include <qtest.h>
+#include "utils.h"
+
+#include <QDebug>
+#include <QTest>
+
 QTEST_MAIN(RecurTodoTest)
 
 using namespace KCalCore;
@@ -30,12 +33,12 @@ void RecurTodoTest::testAllDay()
 {
     qputenv("TZ", "GMT");
     const QDate currentDate = QDate::currentDate();
-    const KDateTime currentUtcDateTime = KDateTime::currentUtcDateTime();
+    const QDateTime currentUtcDateTime = QDateTime::currentDateTimeUtc();
 
     const QDate dueDate(QDate::currentDate());
     Todo *todo = new Todo();
-    todo->setDtStart(KDateTime(dueDate.addDays(-1)));
-    todo->setDtDue(KDateTime(dueDate));
+    todo->setDtStart(QDateTime(dueDate.addDays(-1), {}));
+    todo->setDtDue(QDateTime(dueDate, {}));
     todo->setSummary(QStringLiteral("All day event"));
     todo->setAllDay(true);
 
@@ -44,7 +47,7 @@ void RecurTodoTest::testAllDay()
     Recurrence *recurrence = todo->recurrence();
     recurrence->unsetRecurs();
     recurrence->setDaily(1);
-    QVERIFY(todo->dtDue() == KDateTime(dueDate));
+    QCOMPARE(todo->dtDue(), QDateTime(dueDate, {}));
     todo->setCompleted(currentUtcDateTime);
     QVERIFY(todo->recurs());
     QVERIFY(todo->percentComplete() == 0);
@@ -74,8 +77,8 @@ void RecurTodoTest::testRecurrenceStart()
     Recurrence *recurrence = todo->recurrence();
     recurrence->unsetRecurs();
     recurrence->setDaily(1);
-    todo->setDtStart(KDateTime(fourDaysAgo, currentTime));
-    const KDateTime originalDtDue(treeDaysAgo, currentTime);
+    todo->setDtStart(QDateTime(fourDaysAgo, currentTime));
+    const QDateTime originalDtDue(treeDaysAgo, currentTime);
     todo->setDtDue(originalDtDue);
     todo->setSummary(QStringLiteral("Not an all day event"));
     QVERIFY(!todo->allDay());
@@ -94,8 +97,8 @@ void RecurTodoTest::testNonAllDay()
     const QTime currentTime(currentTimeWithMS.hour(), currentTimeWithMS.minute(), currentTimeWithMS.second());
 
     Todo *todo = new Todo();
-    todo->setDtStart(KDateTime(fourDaysAgo, currentTime));
-    const KDateTime originalDtDue(treeDaysAgo, currentTime);
+    todo->setDtStart(QDateTime(fourDaysAgo, currentTime));
+    const QDateTime originalDtDue(treeDaysAgo, currentTime);
     todo->setDtDue(originalDtDue);
     todo->setSummary(QStringLiteral("Not an all day event"));
     QVERIFY(!todo->allDay());
@@ -103,25 +106,23 @@ void RecurTodoTest::testNonAllDay()
     recurrence->unsetRecurs();
     recurrence->setDaily(1);
     QVERIFY(recurrence->startDateTime().isValid());
-    QVERIFY(todo->dtDue() == originalDtDue);
-    todo->setCompleted(KDateTime::currentUtcDateTime());
+    QCOMPARE(todo->dtDue(), originalDtDue);
+    todo->setCompleted(QDateTime::currentDateTimeUtc());
     QVERIFY(todo->recurs());
     QVERIFY(todo->percentComplete() == 0);
 
-    const bool equal = todo->dtStart() == KDateTime(currentDate,
-                       currentTime,
-                       todo->dtStart().timeSpec()).addDays(1);
+    const bool equal = todo->dtStart() == QDateTime(currentDate, currentTime, todo->dtStart().timeZone()).addDays(1);
     if (!equal) {
         qDebug() << "Test Failed. dtDue = " << todo->dtDue().toString() << "OriginalDtDue:" << originalDtDue.toString()
-                 <<  "KDateTime:"
-                 << KDateTime(currentDate, currentTime, todo->dtDue().timeSpec()).addDays(1).toString();
+                 <<  "QDateTime:"
+                 << QDateTime(currentDate, currentTime, todo->dtDue().timeZone()).addDays(1).toString();
     }
 
     QVERIFY(equal);
 
-    todo->setCompleted(KDateTime::currentUtcDateTime());
-    QVERIFY(todo->dtStart() == KDateTime(currentDate, currentTime, todo->dtStart().timeSpec()).addDays(2));
-    QVERIFY(todo->dtDue(true /*first ocurrence*/) == KDateTime(treeDaysAgo, currentTime));
+    todo->setCompleted(QDateTime::currentDateTimeUtc());
+    QCOMPARE(todo->dtStart(), QDateTime(currentDate, currentTime, todo->dtStart().timeZone()).addDays(2));
+    QCOMPARE(todo->dtDue(true /*first ocurrence*/), QDateTime(treeDaysAgo, currentTime));
 }
 
 void RecurTodoTest::testIsAllDay()
@@ -129,8 +130,8 @@ void RecurTodoTest::testIsAllDay()
     ;
     KCalCore::Todo::Ptr todo(new KCalCore::Todo());
     todo->setUid(QStringLiteral("todo"));
-    todo->setDtStart(KDateTime(QDate(2013, 03, 10), QTime(10, 0, 0), KDateTime::UTC));
-    todo->setDtDue(KDateTime(QDate(2013, 03, 10), QTime(10, 0, 0), KDateTime::UTC));
+    todo->setDtStart(QDateTime(QDate(2013, 03, 10), QTime(10, 0, 0), Qt::UTC));
+    todo->setDtDue(QDateTime(QDate(2013, 03, 10), QTime(10, 0, 0), Qt::UTC));
     todo->recurrence()->setDaily(1);
     todo->recurrence()->setDuration(2);
     QCOMPARE(todo->allDay(), false);
@@ -138,8 +139,9 @@ void RecurTodoTest::testIsAllDay()
 
     KCalCore::Todo::Ptr allDay(new KCalCore::Todo());
     allDay->setUid(QStringLiteral("todo"));
-    allDay->setDtStart(KDateTime(QDate(2013, 03, 10), KDateTime::Spec(KDateTime::UTC)));
-    allDay->setDtDue(KDateTime(QDate(2013, 03, 10), KDateTime::Spec(KDateTime::UTC)));
+    allDay->setDtStart(QDateTime(QDate(2013, 03, 10), {}, Qt::UTC));
+    allDay->setDtDue(QDateTime(QDate(2013, 03, 10), {}, Qt::UTC));
+    allDay->setAllDay(true);
     allDay->recurrence()->setDaily(1);
     allDay->recurrence()->setDuration(2);
     QCOMPARE(allDay->allDay(), true);
@@ -150,7 +152,7 @@ void RecurTodoTest::testHasDueDate()
 {
     KCalCore::Todo::Ptr todo(new KCalCore::Todo());
     todo->setUid(QStringLiteral("todo"));
-    todo->setDtStart(KDateTime(QDate(2013, 03, 10), QTime(10, 0, 0), KDateTime::UTC));
+    todo->setDtStart(QDateTime(QDate(2013, 03, 10), QTime(10, 0, 0), Qt::UTC));
     todo->recurrence()->setDaily(1);
     todo->recurrence()->setDuration(2);
     QVERIFY(!todo->hasDueDate());
@@ -158,23 +160,23 @@ void RecurTodoTest::testHasDueDate()
 
 void RecurTodoTest::testRecurTodo_data()
 {
-    QTest::addColumn<KDateTime>("dtstart");
-    QTest::addColumn<KDateTime>("dtdue");
+    QTest::addColumn<QDateTime>("dtstart");
+    QTest::addColumn<QDateTime>("dtdue");
 
-    const KDateTime today    = KDateTime::currentDateTime(KDateTime::UTC);
-    const KDateTime tomorrow = today.addDays(1);
-    const KDateTime invalid;
+    // Can't use QDateTime::currentDateTimeUtc() due to milliseconds mismatching
+    const QDateTime today = QDateTime::fromSecsSinceEpoch(QDateTime::currentSecsSinceEpoch(), Qt::UTC);
+    const QDateTime tomorrow = today.addDays(1);
+    const QDateTime invalid;
 
     QTest::newRow("valid dtstart") << today << invalid;
     QTest::newRow("valid dtstart and dtdue") << today << tomorrow;
     QTest::newRow("valid dtdue") << invalid << today;
-
 }
 
 void RecurTodoTest::testRecurTodo()
 {
-    QFETCH(KDateTime, dtstart);
-    QFETCH(KDateTime, dtdue);
+    QFETCH(QDateTime, dtstart);
+    QFETCH(QDateTime, dtdue);
 
     KCalCore::Todo::Ptr todo(new KCalCore::Todo());
     todo->setUid(QStringLiteral("todo"));
@@ -186,7 +188,7 @@ void RecurTodoTest::testRecurTodo()
     QCOMPARE(todo->percentComplete(), 0);
 
     // Recur it
-    todo->setCompleted(KDateTime::currentUtcDateTime());
+    todo->setCompleted(QDateTime::currentDateTimeUtc());
     QCOMPARE(todo->percentComplete(), 0);
 
     if (legacyMode) {
@@ -211,7 +213,7 @@ void RecurTodoTest::testRecurTodo()
 
 void RecurTodoTest::testDtStart()
 {
-    KDateTime start(QDate(2013, 03, 10), QTime(10, 0, 0), KDateTime::UTC);
+    QDateTime start(QDate(2013, 03, 10), QTime(10, 0, 0), Qt::UTC);
     KCalCore::Todo::Ptr todo(new KCalCore::Todo());
     todo->setUid(QStringLiteral("todo"));
     todo->setDtStart(start);
@@ -222,7 +224,7 @@ void RecurTodoTest::testDtStart()
     KCalCore::Todo::Ptr todoWithDue(new KCalCore::Todo());
     todoWithDue->setUid(QStringLiteral("todoWithDue"));
     todoWithDue->setDtStart(start);
-    todoWithDue->setDtDue(KDateTime(start).addSecs(60));
+    todoWithDue->setDtDue(QDateTime(start).addSecs(60));
     todoWithDue->recurrence()->setDaily(1);
     todoWithDue->recurrence()->setDuration(2);
     QCOMPARE(todoWithDue->dtStart(), start);
@@ -230,8 +232,8 @@ void RecurTodoTest::testDtStart()
 
 void RecurTodoTest::testRecurrenceBasedOnDtStart()
 {
-    const KDateTime dtstart(QDate(2013, 03, 10), QTime(10, 0, 0), KDateTime::UTC);
-    const KDateTime dtdue(QDate(2013, 03, 10), QTime(11, 0, 0), KDateTime::UTC);
+    const QDateTime dtstart(QDate(2013, 03, 10), QTime(10, 0, 0), Qt::UTC);
+    const QDateTime dtdue(QDate(2013, 03, 10), QTime(11, 0, 0), Qt::UTC);
 
     KCalCore::Todo::Ptr todo(new KCalCore::Todo());
     todo->setUid(QStringLiteral("todo"));
@@ -240,15 +242,15 @@ void RecurTodoTest::testRecurrenceBasedOnDtStart()
     todo->recurrence()->setDaily(1);
     todo->recurrence()->setDuration(3);
 
-    QCOMPARE(todo->recurrence()->getNextDateTime(dtstart), KDateTime(dtstart).addDays(1));
-    QCOMPARE(todo->recurrence()->getNextDateTime(KDateTime(dtstart).addDays(1)), KDateTime(dtstart).addDays(2));
-    QCOMPARE(todo->recurrence()->getNextDateTime(KDateTime(dtstart).addDays(2)), KDateTime());
+    QCOMPARE(todo->recurrence()->getNextDateTime(dtstart), dtstart.addDays(1));
+    QCOMPARE(todo->recurrence()->getNextDateTime(dtstart.addDays(1)), dtstart.addDays(2));
+    QCOMPARE(todo->recurrence()->getNextDateTime(dtstart.addDays(2)), QDateTime());
 }
 
 //For backwards compatibility only
 void RecurTodoTest::testRecurrenceBasedOnDue()
 {
-    const KDateTime dtdue(QDate(2013, 03, 10), QTime(11, 0, 0), KDateTime::UTC);
+    const QDateTime dtdue(QDate(2013, 03, 10), QTime(11, 0, 0), Qt::UTC);
 
     KCalCore::Todo::Ptr todo(new KCalCore::Todo());
     todo->setUid(QStringLiteral("todo"));
@@ -256,7 +258,7 @@ void RecurTodoTest::testRecurrenceBasedOnDue()
     todo->recurrence()->setDaily(1);
     todo->recurrence()->setDuration(3);
 
-    QCOMPARE(todo->recurrence()->getNextDateTime(dtdue), KDateTime(dtdue).addDays(1));
-    QCOMPARE(todo->recurrence()->getNextDateTime(KDateTime(dtdue).addDays(1)), KDateTime(dtdue).addDays(2));
-    QCOMPARE(todo->recurrence()->getNextDateTime(KDateTime(dtdue).addDays(2)), KDateTime());
+    QCOMPARE(todo->recurrence()->getNextDateTime(dtdue), dtdue.addDays(1));
+    QCOMPARE(todo->recurrence()->getNextDateTime(dtdue.addDays(1)), dtdue.addDays(2));
+    QCOMPARE(todo->recurrence()->getNextDateTime(dtdue.addDays(2)), QDateTime());
 }
