@@ -32,7 +32,7 @@
 #include "occurrenceiterator.h"
 #include "calendar.h"
 #include "calfilter.h"
-#include "helper_p.h"
+#include "utils.h"
 
 #include <QDate>
 
@@ -53,22 +53,22 @@ public:
     }
 
     OccurrenceIterator *q;
-    KDateTime start;
-    KDateTime end;
+    QDateTime start;
+    QDateTime end;
 
     struct Occurrence {
         Occurrence()
         {
         }
 
-        Occurrence(const Incidence::Ptr &i, const KDateTime &recurrenceId, const KDateTime &startDate)
+        Occurrence(const Incidence::Ptr &i, const QDateTime &recurrenceId, const QDateTime &startDate)
             : incidence(i), recurrenceId(recurrenceId), startDate(startDate)
         {
         }
 
         Incidence::Ptr incidence;
-        KDateTime recurrenceId;
-        KDateTime startDate;
+        QDateTime recurrenceId;
+        QDateTime startDate;
     };
     QList<Occurrence> occurrenceList;
     QListIterator<Occurrence> occurrenceIt;
@@ -81,7 +81,7 @@ public:
      */
     bool occurrenceIsHidden(const Calendar &calendar,
                             const Incidence::Ptr &inc,
-                            const KDateTime &occurrenceDate)
+                            const QDateTime &occurrenceDate)
     {
         if ((inc->type() == Incidence::TypeTodo) &&
                 calendar.filter() &&
@@ -108,22 +108,19 @@ public:
                 continue;
             }
             if (inc->recurs()) {
-                QHash<KDateTime, Incidence::Ptr> recurrenceIds;
-                KDateTime incidenceRecStart = inc->dateTime(Incidence::RoleRecurrenceStart);
+                QHash<QDateTime, Incidence::Ptr> recurrenceIds;
+                QDateTime incidenceRecStart = inc->dateTime(Incidence::RoleRecurrenceStart);
+                //const bool isAllDay = inc->allDay();
                 foreach (const Incidence::Ptr &exception, calendar.instances(inc)) {
                     if (incidenceRecStart.isValid()) {
-                        recurrenceIds.insert(exception->recurrenceId().toTimeSpec(incidenceRecStart.timeSpec()), exception);
+                        recurrenceIds.insert(exception->recurrenceId().toTimeZone(incidenceRecStart.timeZone()), exception);
                     }
                 }
-                const bool isAllDay = inc->allDay();
-                const DateTimeList occurrences = inc->recurrence()->timesInInterval(start, end);
+                const auto occurrences = inc->recurrence()->timesInInterval(start, end);
                 Incidence::Ptr incidence(inc), lastInc(inc);
                 qint64 offset(0), lastOffset(0);
-                KDateTime occurrenceStartDate;
-                for(KDateTime recurrenceId : qAsConst(occurrences)) {
-                    //timesInInterval generates always date-times,
-                    //which is not what we want for all-day events
-                    recurrenceId.setDateOnly(isAllDay);
+                QDateTime occurrenceStartDate;
+                for(auto recurrenceId : qAsConst(occurrences)) {
                     occurrenceStartDate = recurrenceId;
 
                     bool resetIncidence = false;
@@ -156,7 +153,7 @@ public:
                     }
                 }
             } else {
-                occurrenceList << Private::Occurrence(inc, KDateTime(), inc->dtStart());
+                occurrenceList << Private::Occurrence(inc, {}, inc->dtStart());
             }
         }
         occurrenceIt = QListIterator<Private::Occurrence>(occurrenceList);
@@ -164,10 +161,6 @@ public:
 };
 //@endcond
 
-static uint qHash(const KDateTime &dt)
-{
-    return qHash(dt.toString());
-}
 
 /**
  * Right now there is little point in the iterator, but:
@@ -181,19 +174,19 @@ static uint qHash(const KDateTime &dt)
  * available data structures.
  */
 OccurrenceIterator::OccurrenceIterator(const Calendar &calendar,
-                                       const KDateTime &start,
-                                       const KDateTime &end)
+                                       const QDateTime &start,
+                                       const QDateTime &end)
     : d(new KCalCore::OccurrenceIterator::Private(this))
 {
     d->start = start;
     d->end = end;
 
-    Event::List events = calendar.rawEvents(start.date(), end.date(), start.timeSpec());
+    Event::List events = calendar.rawEvents(start.date(), end.date(), start.timeZone());
     if (calendar.filter()) {
         calendar.filter()->apply(&events);
     }
 
-    Todo::List todos = calendar.rawTodos(start.date(), end.date(), start.timeSpec());
+    Todo::List todos = calendar.rawTodos(start.date(), end.date(), start.timeZone());
     if (calendar.filter()) {
         calendar.filter()->apply(&todos);
     }
@@ -201,7 +194,7 @@ OccurrenceIterator::OccurrenceIterator(const Calendar &calendar,
     Journal::List journals;
     const Journal::List allJournals = calendar.rawJournals();
     for (const KCalCore::Journal::Ptr &journal : allJournals) {
-        const QDate journalStart = journal->dtStart().toTimeSpec(start.timeSpec()).date();
+        const QDate journalStart = journal->dtStart().toTimeZone(start.timeZone()).date();
         if (journal->dtStart().isValid() &&
                 journalStart >= start.date() &&
                 journalStart <= end.date()) {
@@ -220,8 +213,8 @@ OccurrenceIterator::OccurrenceIterator(const Calendar &calendar,
 
 OccurrenceIterator::OccurrenceIterator(const Calendar &calendar,
                                        const Incidence::Ptr &incidence,
-                                       const KDateTime &start,
-                                       const KDateTime &end)
+                                       const QDateTime &start,
+                                       const QDateTime &end)
     : d(new KCalCore::OccurrenceIterator::Private(this))
 {
     Q_ASSERT(incidence);
@@ -249,12 +242,12 @@ Incidence::Ptr OccurrenceIterator::incidence() const
     return d->current.incidence;
 }
 
-KDateTime OccurrenceIterator::occurrenceStartDate() const
+QDateTime OccurrenceIterator::occurrenceStartDate() const
 {
     return d->current.startDate;
 }
 
-KDateTime OccurrenceIterator::recurrenceId() const
+QDateTime OccurrenceIterator::recurrenceId() const
 {
     return d->current.recurrenceId;
 }
