@@ -46,9 +46,13 @@ DFARS 252.227-7013 or 48 CFR 52.227-19, as applicable.
 
 #include "vobject.h"
 #include <string.h>
-#include <strings.h>
 #include <stdio.h>
 
+#ifdef _WIN32
+#define strncasecmp _strnicmp
+#define strcasecmp _stricmp
+#endif
+
 #define NAME_OF(o)          o->id
 #define VALUE_TYPE(o)           o->valType
 #define STRINGZ_VALUE_OF(o)     o->val.strs
